package me.icymint.libra.sage.model;

import java.lang.reflect.Method;
import java.util.LinkedHashMap;
import java.util.Map;

import me.icymint.libra.sage.model.operator.ClassNotMatchException;
import me.icymint.libra.sage.model.operator.InjectException;
import me.icymint.libra.sage.model.operator.Injector;

import org.apache.log4j.Logger;

/**
 * Sql注入器，每一个注入器对应一个{@link Class<?>}。 注入器的作用就是进行依赖注入，不过是以值注入的方式实现的，而不是构建器注入。
 * 其可以对绑定的类的对象进行依赖注入。
 * 
 * @author Daniel
 * @datetime 2012-11-2上午11:40:43
 * 
 */
public class SqlInjector implements Injector {
	protected Logger logger = Logger.getLogger(getClass());
	private final Class<?> cls;
	private final Map<String, SqlProcedure> pmap = new LinkedHashMap<String, SqlProcedure>();
	private final Map<String, Method> mmap = new LinkedHashMap<String, Method>();
	private final Map<String, SqlFunction> fmap = new LinkedHashMap<String, SqlFunction>();

	public SqlInjector(Class<?> cls) {
		if (cls == null)
			throw new NullPointerException();
		this.cls = cls;
		for (Method m : cls.getMethods()) {
			Class<?>[] cs = m.getParameterTypes();
			if (cs.length == 1 && cs[0].equals(String.class)) {
				mmap.put(m.getName().toLowerCase(), m);
			}
		}
	}

	/**
	 * 绑定Sql函数和该注入器对应的类的注入方法。
	 * 
	 * @param sf
	 * @param value
	 *            方法名称，例如类中存在setStr()这样的方法，其值就是"str"。
	 *            setStrStr()方法可以对应值"str_str"或者"strStr"等。
	 * @throws NoSuchMethodException
	 *             方法不存在，则抛出该违例。
	 */
	public void bind(SqlFunction sf, String value) throws NoSuchMethodException {
		String key = value.toLowerCase().replace("_", "");
		if (mmap.containsKey("set" + key))
			fmap.put(key, sf);
		else {
			logger.error(cls + "没有相关方法set" + value + "(String)");
			throw new NoSuchMethodException();
		}
	}

	/**
	 * 
	 * 绑定Sql存储过程和该注入器对应的类的注入方法。
	 * 
	 * @param sp
	 * @param value
	 *            方法名称，例如类中存在setStr()这样的方法，其值就是"str"。
	 *            setStrStr()方法可以对应值"str_str"或者"strStr"。
	 * @throws NoSuchMethodException
	 *             方法不存在，则抛出该违例。
	 */
	public void bind(SqlProcedure sp, String value)
			throws NoSuchMethodException {
		String key = value.toLowerCase().replace("_", "");
		if (mmap.containsKey("set" + key))
			pmap.put(key, sp);
		else {
			logger.error(cls + "没有相关方法set" + value + "(String)");
			throw new NoSuchMethodException();
		}
	}

	public Map<String, SqlFunction> getFunctionBindings() {
		return fmap;
	}

	public Map<String, SqlProcedure> getProcedureBindings() {
		return pmap;
	}

	/**
	 * 返回支持类。 该方法不同于{@link #getClass()}，其返回的是该注入器负责注入的类。
	 */
	public Class<?> getSupportClass() {
		return cls;
	}

	/**
	 * 注入对象。
	 */
	@Override
	public void inject(Object obj) throws InjectException,
			ClassNotMatchException {
		if (cls.isInstance(obj)) {
			for (String key : pmap.keySet()) {
				try {
					mmap.get("set" + key).invoke(obj, pmap.get(key).execSql());
				} catch (Exception e) {
					logger.error("对象注入失败" + key);
					throw new InjectException();
				}
			}
			for (String key : fmap.keySet()) {
				try {
					mmap.get("set" + key).invoke(obj, fmap.get(key).execSql());
				} catch (Exception e) {
					logger.error("对象注入失败" + key);
					throw new InjectException();
				}
			}
		} else {
			logger.error("对象" + obj + "和注入器的类型" + this.getSupportClass()
					+ "不匹配！");
			throw new ClassNotMatchException();
		}
	}
}
